﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PermutationEncoding {
  [Item("ExhaustiveSwapMoveGenerator", "Generates all possible swap moves from a given permutation.")]
  [StorableClass]
  public class ExhaustiveSwapMoveGenerator : SwapMoveGenerator, IExhaustiveMoveGenerator {
    [StorableConstructor]
    protected ExhaustiveSwapMoveGenerator(bool deserializing) : base(deserializing) { }
    protected ExhaustiveSwapMoveGenerator(ExhaustiveSwapMoveGenerator original, Cloner cloner) : base(original, cloner) { }
    public ExhaustiveSwapMoveGenerator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ExhaustiveSwapMoveGenerator(this, cloner);
    }

    public static SwapMove[] Apply(Permutation permutation) {
      int length = permutation.Length;
      if (length == 1) throw new ArgumentException("ExhaustiveSwapMoveGenerator: There cannot be an Swap move given a permutation of length 1.", "permutation");
      int totalMoves = (length) * (length - 1) / 2;
      SwapMove[] moves = null;
      int count = 0;

      if (permutation.PermutationType == PermutationTypes.RelativeUndirected) {
        if (totalMoves - 3 > 0) {
          moves = new SwapMove[totalMoves - 3];
          for (int i = 0; i < length - 1; i++) {
            for (int j = i + 1; j < length; j++) {
              if (j - i >= length - 2) continue;
              moves[count++] = new SwapMove(i, j);
            }
          }
        } else { // when length is 3 or less, there's actually no difference, but for the sake of not crashing the algorithm create a dummy move
          moves = new SwapMove[1];
          moves[0] = new SwapMove(0, 1);
        }
      } else {
        moves = new SwapMove[totalMoves];
        for (int i = 0; i < length - 1; i++)
          for (int j = i + 1; j < length; j++) {
            moves[count++] = new SwapMove(i, j);
          }
      }
      return moves;
    }

    protected override SwapMove[] GenerateMoves(Permutation permutation) {
      return Apply(permutation);
    }
  }
}
